/* -*-c-*-
 *
 * $Id: reffd.h,v 1.1 1999/07/26 23:33:01 mdw Exp $
 *
 * Reference counted file descriptors
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the `fw' port forwarder.
 *
 * `fw' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * `fw' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with `fw'; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: reffd.h,v $
 * Revision 1.1  1999/07/26 23:33:01  mdw
 * Infrastructure for the new design.
 *
 */

#ifndef REFFD_H
#define REFFD_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <unistd.h>

#include <mLib/sub.h>

/*----- Data structures ---------------------------------------------------*/

typedef struct reffd {
  int fd;
  unsigned ref;
  void (*proc)(void */*p*/);
  void *p;
} reffd;

/*----- Functions provided ------------------------------------------------*/

/* --- @reffd_init@ --- *
 *
 * Arguments:	@int fd@ = file descriptor
 *
 * Returns:	Reference-counted file descriptor object.
 *
 * Use:		Creates a refcounted file descriptor.
 */

extern reffd *reffd_init(int /*fd*/);

/* --- @reffd_handler@ --- *
 *
 * Arguments:	@reffd *r@ = pointer to reference counted filehandle
 *		@void (*proc)(void *p)@ = procedure to call
 *		@void *p@
 *
 * Returns:	---
 *
 * Use:		Sets the reference counted file descriptor to call @proc@
 *		when it is no longer required.
 */

extern void reffd_handler(reffd */*r*/, void (*/*proc*/)(void */*p*/),
			  void */*p*/);

/* --- @reffd_inc@ --- *
 *
 * Arguments:	@reffd *r@ = pointer to reference counted filehandle
 *
 * Returns:	---
 *
 * Use:		Increments the reference count for a file descriptor.
 */

#define REFFD_INC(r) do { (r)->ref++; } while (0)

extern void reffd_inc(reffd */*r*/);

/* --- @reffd_dec@ --- *
 *
 * Arguments:	@reffd *r@ = pointer to reference counted filehandle
 *
 * Returns:	---
 *
 * Use:		Decrements the reference count for a file descriptor.
 */

#define REFFD_DEC(r) do {						\
  reffd *_r = (r);							\
  _r->ref--;								\
  if (_r->ref == 0) {							\
    close(_r->fd);							\
    if (_r->proc)							\
      _r->proc(_r->p);							\
    DESTROY(_r);							\
  }									\
} while (0)

extern void reffd_dec(reffd */*r*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
